#!/usr/bin/env python3

import argparse
import csv
import warnings

def main():
    parser = argparse.ArgumentParser(
        description="Tool to combine districts from two block equivalency csv files. This will ignore district 0 if it exists in either file.")
    parser.add_argument(
        'source-one.csv',
        type=argparse.FileType('r'),
        help="Requires BLOCKID and DISTRICT fields.")
    parser.add_argument(
        'source-two.csv',
        type=argparse.FileType('r'),
        help="Requires BLOCKID and DISTRICT fields.")
    parser.add_argument(
        'output-blocks.csv',
        type=argparse.FileType('w'),
        help="Output file for the block equivalency file with the districts combined.")

    args = parser.parse_args()

    blockIds = set()
    count = 0
    overlappingDistricts = set()


    with vars(args)['output-blocks.csv'] as outputFile, vars(args)['source-one.csv'] as sourceFileOne, vars(args)['source-two.csv'] as sourceFileTwo:
        writer = csv.DictWriter(
            outputFile,
            fieldnames=['BLOCKID','DISTRICT'])
        writer.writeheader()
        readerOne = csv.DictReader(sourceFileOne)
        readerTwo = csv.DictReader(sourceFileTwo)
        for row in readerOne:
            # ignore district 0 rows
            if row['DISTRICT'] == "0":
                continue
            writer.writerow({
                'DISTRICT': row['DISTRICT'],
                'BLOCKID' : row['BLOCKID']
                })
            blockIds.add(row['BLOCKID'])
        for row in readerTwo:
            # ignore district 0 rows
            if row['DISTRICT'] == "0":
                continue
            # check for any blockid dupes between files, only output non-dupes
            elif row['BLOCKID'] in blockIds:
                count += 1
                overlappingDistricts.add(row['DISTRICT'])
            else:
                writer.writerow({
                    'DISTRICT': row['DISTRICT'],
                    'BLOCKID' : row['BLOCKID']
                    })

    if count > 0:
        warnings.warn("Found {} duplicate BLOCKIDs, this means you have districts that overlap between the two files.".format(count))
        warnings.warn("Overlapping districts: {}".format(overlappingDistricts))

main()


#!/usr/bin/env python3

import argparse
import csv
import re

def _parse_ranges(numbers: str):
    for x in numbers.split(','):
        x = x.strip()
        if x.isdigit():
            yield int(x)
        elif '-' in x:
            xr = x.split('-')
            yield from range(int(xr[0].strip()), int(xr[1].strip())+1)
        else:
            yield x

def main():
    parser = argparse.ArgumentParser(
        description="Tool to remove districts from a block equivalency csv. Either use --remove or --keep to determine which districts are in the output.")
    parser.add_argument(
        'blocks.csv',
        type=argparse.FileType('r'),
        help="Requires BLOCKID and DISTRICT fields.")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        '--remove',
        type=str,
        help="List of districts to remove separated by commas. Can include ranges such as 1-10.")
    group.add_argument(
        '--keep',
        type=str,
        help="List of districts to keep separated by commas. Can include ranges such as 1-10.")
    parser.add_argument(
        'output-blocks.csv',
        type=argparse.FileType('w'),
        help="Output file for the block equivalency file with the districts removed.")

    args = parser.parse_args()

    if args.remove is not None:
        keep = False
        districts = set(map(str,_parse_ranges(args.remove)))
    else:
        keep = True
        districts = set(map(str,_parse_ranges(args.keep)))

    with vars(args)['output-blocks.csv'] as outputFile, vars(args)['blocks.csv'] as inputFile:
        writer = csv.DictWriter(
            outputFile,
            fieldnames=['BLOCKID','DISTRICT'])
        writer.writeheader()
        reader = csv.DictReader(inputFile)
        for row in reader:
            if not keep:
                if row['DISTRICT'] not in districts:
                    writer.writerow({
                        'DISTRICT': row['DISTRICT'],
                        'BLOCKID' : row['BLOCKID']
                        })
            else:
                if row['DISTRICT'] in districts:
                    writer.writerow({
                        'DISTRICT': row['DISTRICT'],
                        'BLOCKID' : row['BLOCKID']
                        })

main()



#!/usr/bin/env python3

import argparse
import csv

def main():
    parser = argparse.ArgumentParser(description="Tool to nest districts from a block equivalency csv.")
    parser.add_argument(
        'blocks.csv',
        type=argparse.FileType('r'),
        help="Requires BLOCKID and DISTRICT fields.")
    parser.add_argument(
        'nestings.csv',
        type=argparse.FileType('r'),
        help="First column is the senate district and each following column is a nested house district.")
    parser.add_argument(
        'output-senate-blocks.csv',
        type=argparse.FileType('w'),
        help="Outputs BLOCKID and DISTRICT fields for nested senate districts")

    args = parser.parse_args()

    with vars(args)['nestings.csv'] as nestingsFile:
        nestingReader = csv.reader(nestingsFile)
        houseToSenateMap = {}
        for row in nestingReader:
            for index, district in enumerate(row):
                if index == 0:
                    senate = district
                else:
                    if district in houseToSenateMap:
                        raise ValueError("House district in nesting file maps to multiple senate districts.")
                    houseToSenateMap[district] = senate

    with vars(args)['output-senate-blocks.csv'] as outputFile, vars(args)['blocks.csv'] as inputFile:
        writer = csv.DictWriter(
            outputFile,
            fieldnames=['BLOCKID','DISTRICT'])
        writer.writeheader()
        reader = csv.DictReader(inputFile)
        for row in reader:
            if row['DISTRICT'] in houseToSenateMap:
                writer.writerow({
                    'DISTRICT': houseToSenateMap[row['DISTRICT']],
                    'BLOCKID' : row['BLOCKID']
                    })
main()



from setuptools import setup, find_packages

with open('requirements.txt') as f:
	requirements = f.readlines()

long_description = "A set of tools to aid in creating US legislative district maps."

setup(
		name ="redistricting-tools",
		version ="0.0.5",
		author ="Mike Yin",
		author_email ="mikeyin@mikeyin.org",
		url ="https://github.com/yincrash/redistricting-tools",
		description ="Tools for redistricting.",
		long_description = long_description,
		long_description_content_type ="text/markdown",
		license ="MIT",
		packages = find_packages(),
		entry_points ={
			"console_scripts": [
				"combine-districts = redistrictingtools.combine_districts:main",
				"nest-districts = redistrictingtools.nest_districts:main",
				"remove-districts = redistrictingtools.remove_districts:main"
			]
		},
		classifiers =(
			"Programming Language :: Python :: 3",
			"License :: OSI Approved :: MIT License",
			"Operating System :: OS Independent",
		),
		keywords ="redistricting tools",
		install_requires = requirements,
		zip_safe = False
)
